# -*- coding: utf-8 -*-
# @Time    : 2020/11/27 16:38
# @Author  : suhong
# @File    : smb_test.py
# @Software: PyCharm
# !/usr/local/bin/python3
import os
import socket
import sys
import traceback

import fs.smbfs
from nmb.NetBIOS import NetBIOS
from smb import smb_structs
from smb.SMBConnection import SMBConnection
from smb.smb_structs import OperationFailure

from re_common.baselibrary import IniConfig
from re_common.baselibrary.utils.basedir import BaseDir


# 有的smb使用fs.smbfs无法访问，可以使用smb.SMBConnection

class BaseSmb(object):
    """
    fs.smbfs 建立在 pysmb 之上
    host属性中直接输入ip无效，报错fs.errors.CreateFailed: could not get IP/host pair from '*.*.*.193'，
    包的作者目前也没有解决这个问题。加入主机名正常使用
    """

    def __init__(self, host=None, username=None, password=None,port=139,domain=""):
        self.conn = None
        self.host = host
        self.username = username
        self.password = password
        self.port = port
        self.domain = domain


    def set_uri(self, uri):
        """
        'smb://username:password@SAMBAHOSTNAME:port/share'
        由于不知道如何给SAMBAHOSTNAME这个参数，尝试了没有成功
        :return:
        """
        smb_fs = fs.open_fs(uri)
        return smb_fs

    def set_default(self):
        """
        调用该函数默认链接177
        :return:
        """
        # host必须是(servername,ip),不能只提供ip，servername错误一样有效
        self.host = ("VIPS-001", "192.168.31.184")
        self.username = "caiji_smb_rw"
        self.password = "5joH9yST"
        self.port = 139
        self.domain = ""

    def set_config(self, path, sesc):
        dictsall = IniConfig(path).builder().get_config_dict()
        dicts = dictsall[sesc]
        self.host = dicts['host']
        self.username = dicts['username']
        self.password = dicts['password']
        self.port = dicts['port']
        self.domain = dicts['domain']

    def connect(self):
        """
        设置用户名 密码 进行连接
        :return:
        """
        self.conn = fs.smbfs.SMBFS(host=self.host, username=self.username,
                                   passwd=self.password,
                                   port=self.port,
                                   domain=self.domain)

    def close(self):
        # 关闭链接
        self.conn.close()

    def get_gen_dir(self):
        """
        获取根目录下的文件名
        :return:
        """
        for g in self.conn.listdir(""):
            yield g

    def is_exists(self, pathdir):
        """
        目录是否存在
        :param sPth:
        :return:
        """
        return self.conn.exists(pathdir)

    def is_dir(self, pathdir):
        """
        是否是目录
        :param sPth:
        :return:
        """
        return self.conn.isdir(pathdir)

    def is_file(self, pathfile):
        """
        是否是文件
        :param sPth:
        :return:
        """
        return self.conn.isfile(pathfile)

    def create_dir(self, pathdir):
        """
        创建目录 该方法可以创建多级目录
        :param pathdir:
        :return:
        """
        if not self.is_exists(pathdir):
            self.conn.makedirs(pathdir)

    def get_file_size(self, path):
        """
        获取文件大小
        :param path:
        :return:
        """
        size = 0
        size += self.conn.getsize(path)
        return size

    def download_file(self, srcpath, dstpath):
        """
        下载smb服务器文件到本地
        :param srcpath:本地文件
        :param dstpath: 服务器文件
        :return:
        """
        BaseDir.create_dir(srcpath)
        with open(dstpath, 'wb') as write_file:
            self.conn.download(srcpath, write_file)

    def upload_file(self, srcpath, dstpath):
        """
        上传本地文件到smb服务器文件
        :param srcpath:本地文件
        :param dstpath: 服务器文件
        :return:
        """
        self.create_dir(dstpath)
        with open(srcpath, 'rb') as read_file:
            self.conn.upload(dstpath, read_file)

    def copy_file_to_file_smb(self, filePath, tarPath, overwrite=False):
        """
        该方法只适合smb服务器地址到smb服务器地址
        :param filePath:  文件路径
        :param tarPath: 输出文件路径
        :return:
        """
        assert self.is_exists(filePath), FileNotFoundError("需要copy的文件不存在")
        assert self.conn.isfile(filePath), FileNotFoundError("需要copy的不是一个文件")

        self.conn.copy(filePath, tarPath, overwrite=overwrite)

    def copy_dir_to_dir_smb(self, oldDir, newDir, moudle=0):
        """
        该方法只适合smb服务器地址到smb服务器地址
        olddir和newdir都只能是目录，且newdir必须不存在
        :param oldDir:
        :param newDir:
        :return:
        """
        assert self.is_exists(oldDir), IsADirectoryError(oldDir + "目录不存在")
        assert not self.is_exists(newDir), IsADirectoryError(newDir + "目录存在")
        self.conn.copydir(oldDir, newDir)

    def move_file_to_file_smb(self, src_path, dst_path, overwrite=False):

        assert self.is_exists(src_path), FileNotFoundError("需要move的文件不存在")
        assert self.is_file(dst_path), FileNotFoundError("需要move的不是一个文件")
        self.conn.move(src_path, dst_path, overwrite=overwrite)

    def move_dir_to_dir_smb(self, oldDir, newDir, moudle=0):
        """
        该方法只适合smb服务器地址到smb服务器地址
        olddir和newdir都只能是目录，且newdir必须不存在
        :param oldDir:
        :param newDir:
        :return:
        """
        assert self.is_exists(oldDir), IsADirectoryError(oldDir + "目录不存在")
        assert not self.is_exists(newDir), IsADirectoryError(newDir + "目录存在")
        self.conn.movedir(oldDir, newDir)

    def delete_dir(self, dirpath):
        """
        递归删除smb服务器文件夹
        :param dirpath:
        :return:
        """
        assert self.is_file(dirpath), FileExistsError("该目录是个文件")

        self.conn.removetree(dirpath)

    def delete_file(self, filepath):
        """
       删除smb服务器文件
       :param dirpath:
       :return:
       """
        assert not self.is_file(filepath), FileNotFoundError("该路径不是个文件")

        self.conn.remove(filepath)

    def read_file(self, path, mode="r", return_model="yield"):
        """
        :param mode:
        :param path: caiji/except_client_try.txt
        :return:
        """
        with self.conn.open(path, mode, encoding="utf-8") as f:
            if return_model == "yield":
                for fLine in f:
                    yield fLine.strip()
            if return_model == "all":
                return f.read()

    def write_file(self, path, value, mode="w"):
        with self.conn.open(path, mode, encoding="utf-8") as f:
            f.write(value)


class SmbClient(object):
    """
    优缺点：函数功能丰富，文件下载功能只对普通文件如txt,dat,csv有效，压缩文件无效
    """

    def __init__(self, username, password, my_name, remote_name, ip, port=139, domain='', server_name='caiji'):
        self.username = username
        self.password = password
        self.my_name = my_name
        self.ip = ip
        self.port = port
        self.remote_name = remote_name
        self.conn = None
        self.domain = domain
        self.server_name = server_name

    def set_use_smb2(self, is_use=True):
        """
        在 pysmb 中禁用 SMB2 协议，请在创建SMBConnection实例之前将smb_structs模块中
        的SUPPORT_SMB2标志设置为False
        :param is_use:
        :return:
        """
        smb_structs.SUPPORT_SMB2 = is_use

    def get_bios_name(self, timeout=10):
        """
        想通过ip获取对应计算机名，但没有成功
        """
        bios = None
        try:
            bios = NetBIOS()
            srv_name = bios.queryIPForName(self.ip, timeout=timeout)
            print(srv_name)
            return srv_name
        except:
            traceback.print_exc()
            print(sys.stderr, "Looking up timeout, check remote_smb_ip again!!")
        finally:
            if bios is not None:
                bios.close()
            return []

    def set_config(self, path, sesc):
        dictsall = IniConfig(path).builder().get_config_dict()
        dicts = dictsall[sesc]
        self.ip = dicts['ip']
        self.port = dicts['port']
        self.username = dicts['username']
        self.password = dicts['password']
        self.my_name = dicts['my_name']
        self.remote_name = dicts['remote_name']
        self.domain = dicts['domain']

    def connect(self):
        """
        建立smb服务连接
        port: 445或者139
        :return:
        """
        try:
            self.conn = SMBConnection(self.username, self.password, self.my_name,
                                      self.remote_name,
                                      domain=self.domain)
            self.conn.connect(self.ip, self.port)
            status = self.conn.auth_result
        except:
            self.conn.close()
            status = False
        return status

    def all_shares_name(self):
        """
        列出smb服务器下的所有共享目录
        :return:
        """
        share_names = list()
        sharelist = self.conn.listShares()
        for s in sharelist:
            share_names.append(s.name)
        return share_names

    def all_file_names_in_dir(self, service_name, dir_name):
        """
        列出文件夹内所有文件名
        :param service_name: 服务名（smb中的文件夹名，一级目录）
        :param dir_name: 二级目录及以下的文件目录
        :return:
        """
        f_names = list()
        for e in self.conn.listPath(service_name, dir_name):
            if e.filename[0] != '.':  # （会返回一些.的文件，需要过滤）
                f_names.append(e.filename)
        return f_names

    def get_last_updatetime(self, service_name, file_path):
        '''
        返回samba server上的文件更新时间（时间戳），如果出现OperationFailure说明无此文件，返回0
        :param samba:
        :param service_name:
        :param file_path:
        :return:
        '''
        try:
            sharedfile_obj = self.conn.getAttributes(service_name, file_path)
            return sharedfile_obj.last_write_time
        except OperationFailure:
            return 0

    def download(self, f_names, service_name, smb_dir, local_dir):
        """
        下载文件
        :param f_names:文件名
        :param service_name:服务名（smb中的文件夹名）
        :param smb_dir: smb文件夹
        :param local_dir: 本地文件夹
        :return:
        """
        assert isinstance(f_names, list)
        for f_name in f_names:
            f = open(os.path.join(local_dir, f_name), 'wb')
            self.conn.retrieveFile(service_name, os.path.join(smb_dir, f_name), f)
            f.close()

    def createDir(self, service_name, path):
        """
        创建文件夹
        :param samba:
        :param service_name:
        :param path:
        :return:
        """
        try:
            self.conn.createDirectory(service_name, path)
        except OperationFailure:
            pass

    def upload(self, service_name, smb_dir, local_dir, f_names):
        """
        上传文件
        :param samba:
        :param service_name:服务名（smb中的文件夹名）
        :param smb_dir: smb文件夹
        :param local_dir: 本地文件列表所在目录
        :param f_names: 本地文件列表
        :return:
        """
        assert isinstance(f_names, list)
        for f_name in f_names:
            f = open(os.path.join(local_dir, f_name), 'rb')
            self.conn.storeFile(service_name, os.path.join(smb_dir, f_name), f)  # 第二个参数path包含文件全路径
            f.close()


# if __name__ == '__main__':
#     bs = BaseSmb()
#     # bs.set_uri("smb://caiji_smb_rw:5joH9yST@('VIPS-001','192.168.31.184'):139/caiji/except_client_try.txt")
#     bs.set_default()
#     bs.connect()
#     bs.write_file("caiji/except_client_try.txt","aaaa")
#     # for i in bs.get_gen_dir():
#     #     print(i)
#     # print(bs.delete_dir("down_data/test.txt"))

# if __name__ == '__main__':
#     sc = SmbClient("caiji_smb_rw", "5joH9yST", "TEST","VIPS-001", "192.168.31.184", 139, "WorkGroup")
#     sc.connect()





from re_common.vip.baseencodeid import BaseLngid


def encode_id():
    """"
    由 sub_db_id 和 rawid 得到 lngid。
    case_insensitive: 标识源网站的 rawid 是否区分大小写
    """

    sub_db_id = "1"
    rawid = "A44cbe4375b431741"
    b = BaseLngid()
    print("********区分大小写************")
    lngid = b.GetLngid(sub_db_id,
                       rawid,
                       case_insensitive=True)
    print(lngid)
    print("********不区分大小写************")
    lngid = b.GetLngid(sub_db_id,
                       rawid,
                       case_insensitive=False)
    print(lngid)

def decode_id():
    """
    limited_id 是由 lngid去除sub_db_id后得到的字符串
    limited_id超过20长度时，为不可逆的
    :return:
    """
    limited_id_big = "F65F41FFCF049A3A21516C5CDFE40A22"
    limited_id_small = "HLXMH9XNH5XMJC04HHXMJDG2J919Y"
    limited_id_err = "123456" * 20
    b = BaseLngid()
    print("********区分大小写************")
    rawid = b.GetRawid(limited_id_big,
                       case_insensitive=True)
    print(rawid)
    print("********不区分大小写************")
    rawid = b.GetRawid(limited_id_small,
                       case_insensitive=False)
    print(rawid)
    print("*******limited_id超过20长度时*******")
    rawid = b.GetRawid(limited_id_err,
                       case_insensitive=False)





if __name__ == '__main__':
    # encode_id()
    decode_id()


import pickle

import ahocorasick


class ACTool(object):

    def __init__(self):
        self.automaton = ahocorasick.Automaton()

    def add_word(self, key, value, overwrite=True) -> bool:
        """
        为 AC 机添加数据,默认情况下 key重复直接覆盖
        :param key: 要添加的关键字
        :param value: 对应的值
        :param overwrite: 是否覆盖已有的 key，默认为 True
        :return: 是否成功添加或覆盖
        """
        if key in self.automaton:  # 检查 key 是否已存在
            if overwrite:  # 如果允许覆盖
                self.automaton.add_word(key, value)
                return True
            else:  # 不允许覆盖，跳过
                return False
        else:  # key 不存在，直接添加
            self.automaton.add_word(key, value)
            return True

    def is_exists_key(self, key) -> bool:
        # 是否存在key
        if self.automaton.exists(key):
            return True
        else:
            return False

    def make_automaton(self):
        """
        添加完词后需要构建
        """
        self.automaton.make_automaton()

    def iter(self, key):
        """
        结果为可迭代对象 可通过list 转换 [(end_index, value)]
        tool.add_word("he", "word1")
        tool.add_word("hello", "word2")

        # 在字符串中查找匹配
        input_string = "hello world"
        matches = list(tool.automaton.iter(input_string))
        print(matches)  # [(1, 'word1'), (4, 'word2')]

        (1, 'word1'):
        end_index = 1: 表示匹配的关键字 "he" 在 input_string = "hello world" 中的结束位置是索引 1（即字符串 "he" 的最后一个字符 'e' 的位置）。
        "hello world" 的索引：h(0)e(1)l(2)l(3)o(4) (5)w(6)o(7)r(8)l(9)d(10)。
        value = 'word1': 表示匹配的关键字 "he" 对应的值是 "word1"。
        (4, 'word2'):
        end_index = 4: 表示匹配的关键字 "hello" 在 input_string = "hello world" 中的结束位置是索引 4（即字符串 "hello" 的最后一个字符 'o' 的位置）。
        value = 'word2': 表示匹配的关键字 "hello" 对应的值是 "word2"。

        注意: 结果只会返回 value 不会返回 key，如果需要key  请将key 组合到结果中
        """

        result_iter = self.automaton.iter(key)  # ahocorasick.AutomatonSearchIter
        return result_iter
    def save(self,local_temp_path):
        """
        将构建好的ac自动机保存到本地
        """
        self.automaton.save(local_temp_path,pickle.dumps)

    def load(self,local_temp_path):
        """
        加载已经构建好的ac自动机
        """
        self.automaton=ahocorasick.load(local_temp_path, pickle.loads)


# -*- coding:utf-8 -*-
# @Time : 2021/12/15 14:09
# @Author: suhong
# @File : TransformThesisTitleToZt.py
# @Function :转换博硕a层到智图代码
from re_common.baselibrary.utils.basetime import BaseTime

from re_common.facade.mysqlfacade import MysqlUtiles


class TransformThesisTitleToZt():
    def __init__(self):
        # 初始化fields
        self.fields = [
            "lngid",
            "rawid",
            "title",
            "title_alternative",
            "title_sub",
            "identifier_doi",
            "creator",
            "creator_en",
            "creator_bio",
            "creator_degree",
            "creator_discipline",
            "creator_institution",
            "contributor",
            "description",
            "description_en"
            "subject",
            "subject_en",
            "subject_clc",
            "subject_esc",
            "subject_dsa",
            "date",
            "date_created",
            "provider",
            "provider_url",
            "provider_id",
            "description_fund",
            "page",
            "beginpage",
            "endpage",
            "jumppage",
            "pagecount",
            "batch",
            "type",
            "rawtype",
            "medium",
            "language",
            "country",
            "provider_subject",
            "identifier_pisbn",
            "identifier_eisbn",
            "price",
            "pub_place",
            "is_deprecated"
        ]
        self.zt_providermap = dict()
        self.mysqlutils = MysqlUtiles(None, None, builder="MysqlBuilderForDicts", dicts={
            "host": "192.168.31.24",
            "user": "root",
            "passwd": "vipdatacenter",
            "db": "data_warehouse_sql",
            "port": "3306",
            "chartset": "utf8mb4",
        })
        self.get_zt_provider()

    def get_zt_provider(self):
        rows = self.mysqlutils.SelectFromDB(
            "select sub_db_id,provider from a_transform_task where source_type = '4' and out_type = 'zt'")
        for row in rows[1]:
            self.zt_providermap[row[0]] = row[1]

    def transform(self, titleMap):
        transMap = dict()
        sub_db_id = titleMap.get("sub_db_id", "")
        transMap["lngid"] = titleMap.get("lngid", "")
        transMap["rawid"] = titleMap.get("rawid", "")
        transMap["title"] = titleMap.get("title", "")
        transMap["title_alternative"] = titleMap.get("title_alt", "")
        transMap["title_sub"] = titleMap.get("title_sub", "")
        transMap["identifier_doi"] = titleMap.get("doi", "")
        transMap["creator"] = titleMap.get("author", "")
        transMap["creator_en"] = titleMap.get("author_alt", "")
        transMap["creator_bio"] = titleMap.get("author_intro", "")
        transMap["creator_degree"] = titleMap.get("degree", "")
        transMap["creator_discipline"] = titleMap.get("subject_major", "")
        if transMap["creator_discipline"] == "":
            transMap["creator_discipline"] = titleMap.get("subject_dsa", "")
        transMap["creator_institution"] = titleMap.get("organ", "")
        transMap["contributor"] = titleMap.get("contributor", "")
        transMap["description"] = titleMap.get("abstract", "")
        transMap["description_en"] = titleMap.get("abstract_alt", "")
        transMap["subject"] = titleMap.get("keyword", "")
        transMap["subject_en"] = titleMap.get("keyword_alt", "")
        transMap["subject_clc"] = titleMap.get("clc_no", "")
        transMap["subject_esc"] = titleMap.get("subject_edu", "")
        transMap["date"] = titleMap.get("pub_year", "")
        transMap["date_created"] = titleMap.get("pub_date", "")
        # transMap["provider"] = titleMap.get("zt_provider", "")
        # if transMap["provider"] == "":
        transMap["provider"] = self.zt_providermap[sub_db_id]
        transMap["provider_url"] = transMap["provider"] + "@" + titleMap.get("provider_url")
        transMap["provider_id"] = transMap["provider"] + "@" + titleMap.get("rawid")
        transMap["description_fund"] = titleMap.get("fund", "")
        transMap["page"] = titleMap.get("page_info", "")
        transMap["beginpage"] = titleMap.get("begin_page", "")
        transMap["endpage"] = titleMap.get("end_page", "")
        transMap["jumppage"] = titleMap.get("jump_page", "")
        transMap["pagecount"] = titleMap.get("page_cnt", "")
        transMap["batch"] = BaseTime().get_beijin_date_strins("%Y%m%d") + "00"
        transMap["type"] = titleMap.get("source_type", "")
        transMap["rawtype"] = titleMap.get("raw_type", "")
        transMap["medium"] = "2"
        transMap["country"] = titleMap.get("country", "")
        transMap["language"] = titleMap.get("language", "")
        transMap["provider_subject"] = titleMap.get("sub_db_class_name", "")
        transMap["identifier_pisbn"] = titleMap.get("isbn", "")
        transMap["identifier_eisbn"] = titleMap.get("eisbn", "")
        transMap["price"] = titleMap.get("price", "")
        transMap["pub_place"] = titleMap.get("pub_place", "")
        transMap["is_deprecated"] = titleMap.get("is_deprecated", "")

        for field in self.fields:
            if field not in transMap.keys():
                transMap[field] = ""
        return transMap

if __name__ == '__main__':
    t = TransformThesisTitleToZt()






import numpy as np
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.neighbors import NearestNeighbors
import time
import matplotlib.pyplot as plt
import seaborn as sns
import pandas as pd
from sklearn.metrics.pairwise import cosine_similarity

class TextMatcher:
    """
    高性能文本匹配器
    基于 TF-IDF + 最近邻搜索实现相似文献查找
    """
    
    def __init__(self, algorithm='brute', metric='cosine', n_jobs=-1):
        """
        初始化文本匹配器
        
        参数:
        algorithm: 搜索算法 ('brute', 'kd_tree', 'ball_tree', 'lshf')
        metric: 距离度量 ('cosine', 'euclidean', 'manhattan')
        n_jobs: 并行作业数 (-1 表示使用所有CPU核心)
        """
        self.vectorizer = TfidfVectorizer(
            max_features=10000,  # 限制特征数量以提高性能
            stop_words='english',  # 移除英文停用词
            ngram_range=(1, 2)  # 使用单字和双字组合
        )
        
        self.nn = NearestNeighbors(
            algorithm=algorithm,
            metric=metric,
            n_jobs=n_jobs  # 并行处理加速搜索
        )
        
        self.corpus = None
        self.corpus_size = 0
    
    def fit(self, corpus):
        """
        训练匹配器
        """
        self.corpus = corpus
        self.corpus_size = len(corpus)
        print(f"处理 {self.corpus_size} 篇文献...")
        
        # 向量化文本
        start_time = time.time()
        X = self.vectorizer.fit_transform(corpus)
        vectorization_time = time.time() - start_time
        print(f"TF-IDF 向量化完成, 耗时: {vectorization_time:.4f}秒")
        print(f"特征维度: {X.shape[1]}")
        
        # 训练最近邻模型
        start_time = time.time()
        self.nn.fit(X)
        training_time = time.time() - start_time
        print(f"最近邻模型训练完成, 耗时: {training_time:.4f}秒")
        
        return self
    
    def search(self, query, n=5, return_scores=True):
        """
        查找相似文献
        
        参数:
        query: 查询文本
        n: 返回最相似文献的数量
        return_scores: 是否返回相似度分数
        
        返回:
        匹配的文献索引和相似度分数
        """
        if self.corpus is None:
            raise ValueError("请先使用 fit() 方法训练模型")
        
        # 向量化查询文本
        query_vec = self.vectorizer.transform([query])
        
        # 查找最近邻
        start_time = time.time()
        distances, indices = self.nn.kneighbors(query_vec, n_neighbors=n)
        search_time = time.time() - start_time
        
        print(f"搜索完成, 耗时: {search_time:.6f}秒")
        
        # 将距离转换为相似度 (余弦距离 = 1 - 余弦相似度)
        similarities = 1 - distances
        
        # 返回结果
        if return_scores:
            return indices[0], similarities[0]
        return indices[0]
    
    def explain_match(self, query, index):
        """
        解释匹配结果 - 显示查询和匹配文献的关键词
        """
        # 获取TF-IDF特征名
        feature_names = self.vectorizer.get_feature_names_out()
        
        # 向量化查询和匹配文献
        query_vec = self.vectorizer.transform([query])
        doc_vec = self.vectorizer.transform([self.corpus[index]])
        
        # 获取重要特征
        query_data = zip(feature_names, query_vec.toarray()[0])
        doc_data = zip(feature_names, doc_vec.toarray()[0])
        
        # 筛选非零特征
        query_keywords = [(word, score) for word, score in query_data if score > 0]
        doc_keywords = [(word, score) for word, score in doc_data if score > 0]
        
        # 按重要性排序
        query_keywords.sort(key=lambda x: x[1], reverse=True)
        doc_keywords.sort(key=lambda x: x[1], reverse=True)
        
        # 打印结果
        print(f"\n匹配文献 #{index} 解释:")
        print(f"查询关键词: {[word for word, _ in query_keywords[:10]]}")
        print(f"文献关键词: {[word for word, _ in doc_keywords[:10]]}")
        
        # 计算共同关键词
        common_keywords = set([word for word, _ in query_keywords[:20]]) & set([word for word, _ in doc_keywords[:20]])
        print(f"共同关键词: {list(common_keywords)}")
        
        return common_keywords

# ======================
# 演示使用
# ======================

if __name__ == "__main__":
    # 1. 准备文献库 (实际应用中可从文件/数据库加载)
    corpus = [
        "机器学习是人工智能的一个分支，专注于开发算法让计算机从数据中学习",
        "深度学习是机器学习的一个子领域，使用多层神经网络处理复杂模式",
        "自然语言处理(NLP)使计算机能够理解、解释和生成人类语言",
        "计算机视觉关注如何让计算机从图像和视频中获得高层次的理解",
        "强化学习是一种机器学习方法，智能体通过与环境互动学习最优行为策略",
        "监督学习使用标记数据训练模型，无监督学习则处理未标记数据",
        "神经网络是受人脑启发的计算模型，由相互连接的节点层组成",
        "卷积神经网络(CNN)特别适合处理图像识别任务",
        "循环神经网络(RNN)设计用于处理序列数据，如文本和时间序列",
        "Transformer模型通过自注意力机制处理序列数据，成为NLP的主流架构",
        "生成对抗网络(GAN)由生成器和判别器组成，用于生成新数据样本",
        "迁移学习允许将在一个任务上学到的知识应用到另一个相关任务",
        "数据挖掘是从大型数据集中发现模式、关联和异常的过程",
        "特征工程是创建更好的输入特征以提高模型性能的过程",
        "过拟合发生在模型过于复杂，过度记忆训练数据而泛化能力差",
        "正则化技术如L1/L2正则化用于防止过拟合",
        "梯度下降是优化神经网络权重的主要算法",
        "反向传播是训练神经网络的关键算法，用于计算梯度",
        "激活函数如ReLU引入非线性，使神经网络能够学习复杂模式",
        "批量归一化通过标准化层输入加速训练并提高稳定性"
    ]
    
    # 2. 创建文本匹配器
    print("="*50)
    print("创建文本匹配器")
    print("="*50)
    matcher = TextMatcher(
        algorithm='brute',  # 对于小数据集，暴力搜索足够快
        n_jobs=-1           # 使用所有CPU核心
    )
    
    # 3. 训练模型
    matcher.fit(corpus)
    
    # 4. 执行查询
    print("\n" + "="*50)
    print("执行查询: '神经网络在人工智能中的应用'")
    print("="*50)
    query = "神经网络在人工智能中的应用"
    indices, similarities = matcher.search(query, n=3)
    
    # 5. 显示结果
    print("\n最相似的文献:")
    for rank, (idx, sim) in enumerate(zip(indices, similarities)):
        print(f"\nTop {rank+1} [相似度: {sim:.4f}]:")
        print(f"文献 #{idx}: {corpus[idx]}")
        
        # 解释匹配
        matcher.explain_match(query, idx)
    
    # 6. 性能测试 (可选)
    print("\n" + "="*50)
    print("性能测试")
    print("="*50)
    
    # 测试不同文献库大小的性能
    corpus_sizes = [100, 500, 1000, 5000]
    times = []
    
    for size in corpus_sizes:
        # 创建更大的文献库
        large_corpus = corpus * (size // len(corpus) + 1)
        large_corpus = large_corpus[:size]
        
        # 创建新的匹配器
        test_matcher = TextMatcher(algorithm='brute', n_jobs=-1)
        
        # 测量训练时间
        start_time = time.time()
        test_matcher.fit(large_corpus)
        train_time = time.time() - start_time
        
        # 测量查询时间
        start_time = time.time()
        test_matcher.search(query, n=5)
        search_time = time.time() - start_time
        
        times.append((size, train_time, search_time))
        print(f"文献库大小: {size} | 训练时间: {train_time:.4f}s | 查询时间: {search_time:.6f}s")
    
    # 可视化性能结果
    sizes, train_times, search_times = zip(*times)
    
    plt.figure(figsize=(12, 6))
    
    plt.subplot(1, 2, 1)
    plt.plot(sizes, train_times, 'o-')
    plt.title('训练时间 vs 文献库大小')
    plt.xlabel('文献数量')
    plt.ylabel('时间 (秒)')
    plt.grid(True)
    
    plt.subplot(1, 2, 2)
    plt.plot(sizes, search_times, 'o-')
    plt.title('查询时间 vs 文献库大小')
    plt.xlabel('文献数量')
    plt.ylabel('时间 (秒)')
    plt.grid(True)
    
    plt.tight_layout()
    plt.savefig('performance.png')
    print("\n性能图表已保存为 'performance.png'")
    
    # 7. 相似度矩阵可视化 (可选)
    print("\n" + "="*50)
    print("文献相似度矩阵")
    print("="*50)
    
    # 计算所有文献的TF-IDF向量
    vectors = matcher.vectorizer.transform(corpus)
    
    # 计算余弦相似度矩阵
    sim_matrix = cosine_similarity(vectors)
    
    # 创建DataFrame用于可视化
    df = pd.DataFrame(sim_matrix, 
                     columns=[f"Doc{i}" for i in range(len(corpus))],
                     index=[f"Doc{i}" for i in range(len(corpus))])
    
    plt.figure(figsize=(12, 10))
    sns.heatmap(df, cmap="YlGnBu", annot=False)
    plt.title("文献相似度矩阵")
    plt.tight_layout()
    plt.savefig('similarity_matrix.png')
    print("相似度矩阵已保存为 'similarity_matrix.png'")


# elasticsearch[async]~=8.11.1
from typing import Any, Dict, List
from elasticsearch import AsyncElasticsearch, helpers

DEFAULT_CONFIG = {
    "hosts": [
        "http://192.168.32.97:9200",
        "http://192.168.32.99:9200",
        "http://192.168.32.101:9200",
        "http://192.168.32.103:9200",
    ],
    "basic_auth": ("elastic", "Zl2bWsOuvqi0IUwpvGhK"),
    "timeout": 60,
}


def get_es(config=DEFAULT_CONFIG):
    es = AsyncElasticsearch(**config)
    return es


async def update(es: AsyncElasticsearch, index: str, doc_id: str, doc: Dict[str, Any], doc_as_upsert: bool = False):
    return await es.update(index=index, id=doc_id, doc=doc, doc_as_upsert=doc_as_upsert)


async def bulk_update(es: AsyncElasticsearch, index: str, docs: List[Dict[str, Any]], doc_as_upsert: bool = False):
    """
    批量更新ES文档

    docs 格式示例：
    [
        {"_id": "1", "doc": {"field1": "value1"}},
        {"_id": "2", "doc": {"field2": "value2"}},
    ]
    """
    actions = []
    for item in docs:
        action = {
            "_op_type": "update",
            "_index": index,
            "_id": item["_id"],
            "doc": item["doc"],
            "doc_as_upsert": doc_as_upsert,
        }
        actions.append(action)
    return await helpers.async_bulk(es, actions=actions)



import platform
import traceback

import win32file

from re_common.baselibrary.utils.basefile import BaseFile


# aa = r"F:\cnipa_ftp\SIPO\CN-PA-IMGS-10-A 中国发明专利申请公布标准化全文图像数据\20211109\20211109-1-002.ZIP"
# aa = r"F:\cnipa_ftp\SIPO\CN-PA-IMGS-10-A 中国发明专利申请公布标准化全文图像数据\20211109\20211109-1-001.ZIP"
def is_used(file_name):
    if "Windows" == platform.system():
        import win32file
        try:
            vHandle = win32file.CreateFile(file_name, win32file.GENERIC_READ, 0, None, win32file.OPEN_EXISTING,
                                           win32file.FILE_ATTRIBUTE_NORMAL, None)
            return int(vHandle) == win32file.INVALID_HANDLE_VALUE
        except:
            if "另一个程序正在使用此文件，进程无法访问。" in traceback.format_exc():
                return True
            else:
                return False
        finally:
            try:
                win32file.CloseHandle(vHandle)
            except:
                pass
    else:
        raise Exception("不是windows系统，请不要调用该函数判断文件是否被打开")

aa = is_used(r"\\192.168.31.171\caiji\test.txt")
print(aa)


from re_common.baselibrary.utils.baseodbc import BaseODBC

baseodbc = BaseODBC(r"C:\Users\xuzhu\Desktop\DB_20200701_GB.mdb")
# baseodbc = BaseODBC(r"D:\download\cnki_qk\download\get_journal\mdb\cnki期刊信息_20200315.mdb")
baseodbc.get_cur()
sql = "select * from `CN`"
for row in baseodbc.select(sql):
    print(row)